/* ---------------------------------------------------- *
 * hrandom_test.c - how to use /dev/hrandom             *
 * ---------------------------------------------------- *
 * Copyright (c) 2006 - Olivier Rochecouste             *
 *                                                      *
 * This program is free software; you can redistribute  *
 * it and/or modify it under the terms of the GNU       *
 * General Public License as published by the Free      *
 * Software Foundation; either version 2 of the License,*
 * or (at your option) any later version.               *
 *                                                      *
 * This program is distributed in the hope that it will *
 * be useful, but WITHOUT ANY WARRANTY; without even the*
 * implied warranty of MERCHANTABILITY or FITNESS FOR A *
 * PARTICULAR PURPOSE. See the GNU General Public       *
 * License for more details.                            *
 *                                                      *
 * You should have received a copy of the GNU General   *
 * Public License along with this program; if not, write*
 * to the Free Software Foundation, Inc., 675 Mass Ave, *
 * Cambridge, MA 02139, USA.                            *
 *                                                      *
 * contact information: orocheco [at] irisa [dot] fr    *
 * ==================================================== */

#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

/* -------------------- *
 * hrandom device nodes *
 * -------------------- */

#define HRANDOM0 "/dev/hrandom0"
#define HRANDOM1 "/dev/hrandom1"

#define _04MB  4000*1024
#define _32MB 32000*1024
static int pool [_32MB];

/* --------------------------------------------------- *
 * dump the content of the random pool in file "fname" *
 * --------------------------------------------------- */

#ifdef DUMP_CONTENT
void
dump_bin (char *fname, int *buf, int size)
{
  FILE *fout;

  if (!(fout = fopen (fname, "wb"))) {
    fprintf(stderr, "cannot open file <%s> for writing.\n",
	    fname);
    return;
  }

  if (fwrite (buf, sizeof (int), size, fout) == 0) {
    fprintf(stderr, "cannot write data in file: %s.\n",
	    fname);
    return;
  }

  fclose (fout);
}
#endif

int
main (int argc, char *argv[])
{
  int fd, err;

#ifdef DUMP_CONTENT
  char *fname = "hrandom0.out";
#endif

  /* open hrandm in read-only mode */
  fd = open (HRANDOM0, O_RDONLY);
  if (fd == ENODEV) {
    fprintf (stderr, "cannot find: %s.\n", HRANDOM0);
    exit (1);
  }

  /* get random data from HRANDOM0 */
  err = read (fd, pool, _32MB);
  if (err != _32MB) {
    fprintf (stderr, "cannot read %d bytes from %s.\n",
	     _32MB, HRANDOM0);
    exit (1) ;
  }

#ifdef DUMP_CONTENT
  /* dump content of random pool */
  dump_bin (fname, pool, _32MB/sizeof(int));
#endif

#ifdef RUN_NIST
  /* perform NIST tests on the first _04MB of data */
  if (PackTestF (pool, _04MB, "ndrand_nist1.out") < 8) {
    PackTestL (pool, _04MB, "ndrand_nist1.out");
  }
#endif
}
